# Generated by roxygen2: do not edit by hand

S3method(as.col_spec,"NULL")
S3method(as.col_spec,character)
S3method(as.col_spec,col_spec)
S3method(as.col_spec,default)
S3method(as.col_spec,list)
S3method(collector_value,collector_character)
S3method(collector_value,collector_date)
S3method(collector_value,collector_datetime)
S3method(collector_value,collector_double)
S3method(collector_value,collector_factor)
S3method(collector_value,collector_guess)
S3method(collector_value,collector_integer)
S3method(collector_value,collector_logical)
S3method(collector_value,collector_numeric)
S3method(collector_value,collector_time)
S3method(output_column,POSIXt)
S3method(output_column,character)
S3method(output_column,default)
S3method(output_column,double)
S3method(output_column,factor)
S3method(print,vroom_altrep)
S3method(summary,col_spec)
S3method(vroom_str,data.frame)
S3method(vroom_str,default)
export(as.col_spec)
export(col_big_integer)
export(col_character)
export(col_date)
export(col_datetime)
export(col_double)
export(col_factor)
export(col_guess)
export(col_integer)
export(col_logical)
export(col_number)
export(col_skip)
export(col_time)
export(cols)
export(cols_condense)
export(cols_only)
export(contains)
export(date_names)
export(date_names_lang)
export(date_names_langs)
export(default_locale)
export(ends_with)
export(everything)
export(fwf_cols)
export(fwf_empty)
export(fwf_positions)
export(fwf_widths)
export(gen_character)
export(gen_date)
export(gen_datetime)
export(gen_double)
export(gen_factor)
export(gen_integer)
export(gen_logical)
export(gen_name)
export(gen_number)
export(gen_tbl)
export(gen_time)
export(guess_type)
export(last_col)
export(locale)
export(matches)
export(num_range)
export(one_of)
export(output_column)
export(problems)
export(spec)
export(starts_with)
export(vroom)
export(vroom_altrep)
export(vroom_altrep_opts)
export(vroom_example)
export(vroom_examples)
export(vroom_format)
export(vroom_fwf)
export(vroom_lines)
export(vroom_progress)
export(vroom_str)
export(vroom_write)
export(vroom_write_lines)
import(rlang)
importFrom(bit64,integer64)
importFrom(crayon,blue)
importFrom(crayon,bold)
importFrom(crayon,cyan)
importFrom(crayon,green)
importFrom(crayon,reset)
importFrom(crayon,silver)
importFrom(glue,glue)
importFrom(lifecycle,deprecate_warn)
importFrom(lifecycle,deprecated)
importFrom(methods,setOldClass)
importFrom(tidyselect,contains)
importFrom(tidyselect,ends_with)
importFrom(tidyselect,everything)
importFrom(tidyselect,last_col)
importFrom(tidyselect,matches)
importFrom(tidyselect,num_range)
importFrom(tidyselect,one_of)
importFrom(tidyselect,starts_with)
useDynLib(vroom, .registration = TRUE)
